from plone.formwidget.recurrence import _
from plone.formwidget.recurrence import pl_message
from zope.i18n import translate


def translations(request):
    translation_table = dict(
        displayUnactivate=translate(_(u'display_unactivate'), context=request),
        displayActivate=translate(_(u'display_activate'), context=request),
        add_rules=translate(_(u'add_rules'), context=request),
        edit_rules=translate(_(u'edit_rules'), context=request),
        delete_rules=translate(_(u'delete_rules'), context=request),
        add=translate(_(u'add'), context=request),
        refresh=translate(_(u'refresh'), context=request),
        title=translate(_(u'title'), context=request),
        preview=translate(_(u'preview'), context=request),
        addDate=translate(_(u'add_date'), context=request),
        recurrenceType=translate(_(u'recurrence_type'), context=request),
        dailyInterval1=translate(_(u'daily_interval_1'), context=request),
        dailyInterval2=translate(_(u'daily_interval_2'), context=request),
        weeklyInterval1=translate(_(u'weekly_interval_1'), context=request),
        weeklyInterval2=translate(_(u'weekly_interval_2'), context=request),
        weeklyWeekdays=translate(_(u'weekly_weekdays'), context=request),
        weeklyWeekdaysHuman=translate(_(u'weekly_weekdays_human'), context=request),
        monthlyInterval1=translate(_(u'monthly_interval_1'), context=request),
        monthlyInterval2=translate(_(u'monthly_interval_2'), context=request),
        monthlyDayOfMonth1=translate(_(u'monthly_day_of_month_1'), context=request),
        monthlyDayOfMonth1Human=translate(_(u'monthly_day_of_month_1_human'), context=request),
        monthlyDayOfMonth2=translate(_(u'monthly_day_of_month_2'), context=request),
        monthlyDayOfMonth3=translate(_(u'monthly_day_of_month_3'), context=request),
        monthlyDayOfMonth4=translate(_(u'monthly_day_of_month_4'), context=request),
        monthlyWeekdayOfMonth1=translate(_(u'monthly_weekday_of_month_1'), context=request),
        monthlyWeekdayOfMonth1Human=translate(_(u'monthly_weekday_of_month_1_human'), context=request),
        monthlyWeekdayOfMonth2=translate(_(u'monthly_weekday_of_month_2'), context=request),
        monthlyWeekdayOfMonth3=translate(_(u'monthly_weekday_of_month_3'), context=request),
        monthlyRepeatOn=translate(_(u'monthly_repeat_on'), context=request),
        yearlyInterval1=translate(_(u'yearly_interval_1'), context=request),
        yearlyInterval2=translate(_(u'yearly_interval_2'), context=request),
        yearlyDayOfMonth1=translate(_(u'yearly_day_of_month_1'), context=request),
        yearlyDayOfMonth1Human=translate(_(u'yearly_day_of_month_1_human'), context=request),
        yearlyDayOfMonth2=translate(_(u'yearly_day_of_month_2'), context=request),
        yearlyDayOfMonth3=translate(_(u'yearly_day_of_month_3'), context=request),
        yearlyWeekdayOfMonth1=translate(_(u'yearly_weekday_of_month_1'), context=request),
        yearlyWeekdayOfMonth1Human=translate(_(u'yearly_weekday_of_month_1_human'), context=request),
        yearlyWeekdayOfMonth2=translate(_(u'yearly_weekday_of_month_2'), context=request),
        yearlyWeekdayOfMonth3=translate(_(u'yearly_weekday_of_month_3'), context=request),
        yearlyWeekdayOfMonth4=translate(_(u'yearly_weekday_of_month_4'), context=request),
        yearlyRepeatOn=translate(_(u'yearly_repeat_on'), context=request),
        range=translate(_(u'range'), context=request),
        rangeNoEnd=translate(_(u'range_no_end'), context=request),
        rangeByOccurrences1=translate(_(u'range_by_occurrences_1'), context=request),
        rangeByOccurrences1Human=translate(_(u'range_by_occurrences_1_human'), context=request),
        rangeByOccurrences2=translate(_(u'range_by_occurrences_2'), context=request),
        rangeByEndDate=translate(_(u'range_by_end_date'), context=request),
        rangeByEndDateHuman=translate(_(u'range_by_end_date_human'), context=request),
        including=translate(_(u'including'), context=request),
        cancel=translate(_(u'cancel'), context=request),
        save=translate(_(u'save'), context=request),
        recurrenceStart=translate(_(u'recurrence_start'), context=request),
        additionalDate=translate(_(u'additional_date'), context=request),
        include=translate(_(u'include'), context=request),
        exclude=translate(_(u'exclude'), context=request),
        remove=translate(_(u'remove'), context=request),
        orderIndexes=[
            translate(_(u'order_indexes_first'), context=request),
            translate(_(u'order_indexes_second'), context=request),
            translate(_(u'order_indexes_third'), context=request),
            translate(_(u'order_indexes_fourth'), context=request),
            translate(_(u'order_indexes_last'), context=request)
        ],
        months=[
            translate(pl_message(u'month_jan'), context=request),
            translate(pl_message(u'month_feb'), context=request),
            translate(pl_message(u'month_mar'), context=request),
            translate(pl_message(u'month_apr'), context=request),
            translate(pl_message(u'month_may'), context=request),
            translate(pl_message(u'month_jun'), context=request),
            translate(pl_message(u'month_jul'), context=request),
            translate(pl_message(u'month_aug'), context=request),
            translate(pl_message(u'month_sep'), context=request),
            translate(pl_message(u'month_oct'), context=request),
            translate(pl_message(u'month_nov'), context=request),
            translate(pl_message(u'month_dec'), context=request)
        ],
        shortMonths=[
            translate(pl_message(u'month_jan_abbr'), context=request),
            translate(pl_message(u'month_feb_abbr'), context=request),
            translate(pl_message(u'month_mar_abbr'), context=request),
            translate(pl_message(u'month_apr_abbr'), context=request),
            translate(pl_message(u'month_may_abbr'), context=request),
            translate(pl_message(u'month_jun_abbr'), context=request),
            translate(pl_message(u'month_jul_abbr'), context=request),
            translate(pl_message(u'month_aug_abbr'), context=request),
            translate(pl_message(u'month_sep_abbr'), context=request),
            translate(pl_message(u'month_oct_abbr'), context=request),
            translate(pl_message(u'month_nov_abbr'), context=request),
            translate(pl_message(u'month_dec_abbr'), context=request)
        ],
        weekdays=[
            translate(pl_message(u'weekday_sun'), context=request),
            translate(pl_message(u'weekday_mon'), context=request),
            translate(pl_message(u'weekday_tue'), context=request),
            translate(pl_message(u'weekday_wed'), context=request),
            translate(pl_message(u'weekday_thu'), context=request),
            translate(pl_message(u'weekday_fri'), context=request),
            translate(pl_message(u'weekday_sat'), context=request)
        ],
        shortWeekdays=[
            translate(pl_message(u'weekday_sun_abbr'), context=request),
            translate(pl_message(u'weekday_mon_abbr'), context=request),
            translate(pl_message(u'weekday_tue_abbr'), context=request),
            translate(pl_message(u'weekday_wed_abbr'), context=request),
            translate(pl_message(u'weekday_thu_abbr'), context=request),
            translate(pl_message(u'weekday_fri_abbr'), context=request),
            translate(pl_message(u'weekday_sat_abbr'), context=request)
        ],
        longDateFormat=translate(_(u'long_date_format'), context=request),
        shortDateFormat=translate(pl_message(u'date_format_short_datepicker'), context=request),
        unsupportedFeatures=translate(_(u'unsupported_features'), context=request),
        noTemplateMatch=translate(_(u'no_template_match'), context=request),
        multipleDayOfMonth=translate(_(u'multiple_day_of_month'), context=request),
        bysetpos=translate(_(u'bysetpos'), context=request),
        noRule=translate(_(u'no_rule'), context=request),
        noRepeatEvery=translate(_(u'no_repeat_every'), context=request),
        noEndDate=translate(_(u'no_end_date'), context=request),
        noRepeatOn=translate(_(u'no_repeat_on'), context=request),
        pastEndDate=translate(_(u'past_end_date'), context=request),
        noEndAfterNOccurrences=translate(_(u'no_end_after_n_occurrences'), context=request),
        alreadyAdded=translate(_(u'already_added'), context=request),
        rtemplate=dict(
            daily=translate(_(u'template_daily'), context=request),
            mondayfriday=translate(_(u'template_mondayfriday'), context=request),
            weekdays=translate(_(u'template_weekdays'), context=request),
            weekly=translate(_(u'template_weekly'), context=request),
            monthly=translate(_(u'template_monthly'), context=request),
            yearly=translate(_(u'template_yearly'), context=request),
        ),
        # a bit wonky here, except is a reserved word
        **{"except": translate(_(u'except'), context=request)}
    )
    return translation_table



from Products.Archetypes.tests.utils import makeContent
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.formwidget.recurrence.tests.base import IntegrationTestCase
from plone.testing.z2 import Browser

import transaction


TESTVALUE = "FREQ=MONTHLY;BYDAY=+3TU;COUNT=5"


class ATWidgetTestCase(IntegrationTestCase):

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.obj = makeContent(
            self.folder, portal_type='RecurrenceType', id='testobj'
        )
        self.field = self.obj.getField('rec')
        self.widget = self.field.widget

        login(self.portal, TEST_USER_NAME)
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        transaction.commit()

    def tearDown(self):
        super(ATWidgetTestCase, self).tearDown()
        self.portal.manage_delObjects(['test-folder'])
        transaction.commit()

    def test_widget_properties(self):
        widget = self.widget
        self.assertEqual(widget.macro_edit, 'recurrence_widget')
        self.assertTrue(widget.startField == 'test_start_field')
        self.assertTrue(widget.startYear == 'test_start_year')
        self.assertTrue(widget.startMonth == 'test_start_month')
        self.assertTrue(widget.startDay == 'test_start_day')
        self.assertTrue(widget.first_day == 4)

    def test_widget_process(self):
        self.assertFalse(self.widget.process_form(self.obj, self.field, {}))
        self.assertEqual(
            self.widget.process_form(self.obj, self.field, {'rec': TESTVALUE}),
            (TESTVALUE, {})
        )

    def test_widget_rendering(self):
        self.browser.addHeader(
            'Authorization', 'Basic %s:%s' %
            (TEST_USER_NAME, TEST_USER_PASSWORD,)
        )
        self.browser.open(self.obj.absolute_url())
        self.browser.getLink('Edit').click()
        self.assertIn('Recurrence', self.browser.contents)



from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from pkg_resources import resource_string
from plone.formwidget.recurrence.browser.i18n import translations


class RecurrenceWidget(BrowserView):
    """ """
    template = ViewPageTemplateFile('widget.pt')

    @property
    def macros(self):
        return self.template.macros

    def translation(self):
        return translations(request=self.request)


class Templates(BrowserView):

    def __call__(self):
        return resource_string(
            'plone.formwidget.recurrence.browser',
            'lib/templates.html'
        )



from Products.Archetypes.atapi import BaseSchema
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import registerType
from Products.Archetypes.examples.SimpleType import SimpleType
from plone.formwidget.recurrence.at.widget import RecurrenceWidget


schema = BaseSchema.copy() + Schema((
    LinesField(
        'rec',
        widget=RecurrenceWidget(
            label='Recurrence',
            startField='test_start_field',
            startYear='test_start_year',
            startMonth='test_start_month',
            startDay='test_start_day',
            first_day=4,
        ),
    ),
))


class RecurrenceType(SimpleType):
    """A simple archetype"""
    schema = schema
    archetype_name = meta_type = "RecurrenceType"
    portal_type = 'RecurrenceType'


registerType(RecurrenceType, 'plone.formwidget.recurrence.tests.at_example')



from OFS.SimpleItem import SimpleItem
from plone.app.z3cform.widget import DateFieldWidget
from plone.formwidget.recurrence.tests.base import IntegrationTestCase
from plone.formwidget.recurrence.z3cform.widget import RecurrenceFieldWidget
from z3c.form import form, field
from z3c.form.testing import TestRequest
from zope.schema.fieldproperty import FieldProperty

import zope.interface
import zope.schema


class ITestForm(zope.interface.Interface):
    recurrence = zope.schema.Text(title=u'Recurrence', required=True)
    day = zope.schema.Date(title=u'Day', required=True)


@zope.interface.implementer(ITestForm)
class TestForm(SimpleItem):
    recurrence = FieldProperty(ITestForm['recurrence'])
    day = FieldProperty(ITestForm['day'])

    def __init__(self, recurrence, day):
        super(TestForm, self).__init__(id)
        self.recurrence = recurrence
        self.day = day


class TestAddForm(form.AddForm):
    fields = field.Fields(ITestForm)
    fields['recurrence'].widgetFactory = RecurrenceFieldWidget
    fields['day'].widgetFactory = DateFieldWidget


class Z3CWidgetTestCase(IntegrationTestCase):

    def setUp(self):
        self.portal = self.layer['portal']

    def test_widget_rendering(self):
        request = TestRequest()
        request.LANGUAGE = 'en'
        form = TestAddForm(self.portal, request)
        form.update()

        widget = RecurrenceFieldWidget(form.fields['recurrence'].field,
                                       request)
        widget.form = form
        widget.start_field = 'day'
        widget.update()

        html = widget.render()
        self.assertIn('recurrence', html)



from Products.Archetypes import atapi

packageName = __name__


def initialize(context):
    """Register content types through Archetypes with Zope and the CMF.
    """
    from Products.CMFCore.utils import ContentInit
    from Products.CMFCore.permissions import AddPortalContent
    from plone.formwidget.recurrence.tests.at_example import content  # noqa

    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(packageName), packageName)

    for atype, constructor, fti in zip(content_types, constructors, ftis):
        ContentInit(
            "%s: %s" % (packageName, atype.portal_type),
            content_types=(atype,),
            permission = AddPortalContent,
            extra_constructors = (constructor,),
            fti = (fti,),
        ).initialize(context)



from App.class_init import InitializeClass
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.atapi import LinesWidget
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_callable
from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator
from dateutil import rrule
from zope.component.hooks import getSite
from zope.interface import implementer
from plone.formwidget.recurrence.browser.i18n import translations

import json


class RecurrenceWidget(LinesWidget):
    _properties = LinesWidget._properties.copy()
    _properties.update({
        'macro_edit': "recurrence_widget",
        'helper_js': (),
        'helper_css': (),
        'startField': '',
        'startFieldYear': '',
        'startFieldMonth': '',
        'startFieldDay': '',
        'first_day': '',
        'show_repeat_forever': True,
    })

    def get_pattern_options(self):
        portal = getToolByName(getSite(), 'portal_url').getPortalObject()
        ajax_url = portal.absolute_url() + '/@@json_recurrence'
        request = portal.REQUEST

        first_day = self.first_day
        first_day = first_day() if safe_callable(first_day) else first_day

        params = dict(
            ajaxContentType='application/x-www-form-urlencoded; charset=UTF-8',
            ajaxURL=ajax_url,
            firstDay=first_day,
            hasRepeatForeverButton=self.show_repeat_forever,
            lang=request.LANGUAGE,
            ributtonExtraClass='allowMultiSubmit',
            startField=self.startField,
            startFieldDay=self.startFieldDay,
            startFieldMonth=self.startFieldMonth,
            startFieldYear=self.startFieldYear,
        )
        return json.dumps({
            "locationization": translations(request),
            "language": request.LANGUAGE,
            "configuration": params
        })


InitializeClass(RecurrenceWidget)
registerWidget(RecurrenceWidget,
               title='Recurring Date',
               description=('Renders a recurrence widget to enter all the '
                            'info for recurring dates.'),
               used_for=('plone.app.event.recurrence.RecurrenceField',))


@implementer(IValidator)
class RecurrenceValidator(object):

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        try:
            rrule.rrulestr(value)   # TODO: rm dep. on rrule. check with regex
            assert('FREQ' in value) # TODO: check if freq before other
                                    # recurrence parms
        except (ValueError, TypeError, AssertionError):
            return "Validation failed: Please enter valid recurrence data."

        return True
validation.register(RecurrenceValidator('isRecurrence'))



from zope import schema
from zope.interface import implementer
from plone.formwidget.recurrence.z3cform.interfaces import IRecurrenceField


@implementer(IRecurrenceField)
class RecurrenceField(schema.Text):
    pass



from setuptools import setup, find_packages


version = '2.1.1'

setup(
    name='plone.formwidget.recurrence',
    version=version,
    description="Recurrence widget for Plone",
    long_description="%s\n%s" % (open("README.rst").read(),
                                 open("CHANGES.rst").read()),
    classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='Plone Event Recurrence Date Time Widget Archetypes z3c.form',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://github.com/plone/plone.formwidget.recurrence',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['plone', 'plone.formwidget'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'Products.CMFCore',
        'Products.CMFPlone',
        'python-dateutil',
        'zope.component',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.interface',
    ],
    extras_require=dict(
        z3cform=[
            'z3c.form',
            'zope.schema',
            'zope.traversing',
        ],
        archetypes=[
            'Products.Archetypes',
            'Products.validation',
        ],
        test=[
            'Products.ATContentTypes',
            'Products.GenericSetup',
            'plone.app.testing',
            'plone.app.z3cform',
            'plone.formwidget.recurrence[archetypes, z3cform]',
        ]
    ),
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from z3c.form.interfaces import IWidget
from zope.schema import ValidationError
from zope.schema.interfaces import IText
from plone.formwidget.recurrence import _


# Fields
class IRecurrenceField(IText):
    """Special marker for datetime fields that use our widget.
    """


# Widgets
class IRecurrenceWidget(IWidget):
    """Date widget marker for z3c.form.
    """


# Errors
class RecurrenceValidationError(ValidationError):
    __doc__ = _(u"The recurrence rule couldn't be parsed.")
